{%- if args -%}
    {%- if 'body' in args %}
    async def {{ interface_method }}(self, {{ args|join(', ') }}) -> Response:
        return await self._request("{{ http_method }}", f"{self.api_url}{{ path }}", json=body)
    {%- else %}
    async def {{ interface_method }}(self, {{ args|join(', ') }}) -> Response:
        return await self._request("{{ http_method }}", f"{self.api_url}{{ path }}")
    {%- endif -%}
{%- else %}
    async def {{ interface_method }}(self) -> Response:
        return await self._request("{{ http_method }}", f"{self.api_url}{{ path }}")
{%- endif %}

