from vedro_httpx import AsyncHTTPInterface, Response


class Api(AsyncHTTPInterface):
    {% if base_url -%}
    def __init__(self, base_url: str = '{{ base_url }}') -> None:
    {%- else -%}
    def __init__(self, base_url: str) -> None:
    {%- endif %}
        super().__init__(base_url)
        self.api_url = base_url

